Package: S7
Title: An Object Oriented System Meant to Become a Successor to S3 and
        S4
Version: 0.2.0
Authors@R: c(
    person("Object-Oriented Programming Working Group", role = "cph"),
    person("Davis", "Vaughan", role = "aut"),
    person("Jim", "Hester", role = "aut",
           comment = c(ORCID = "0000-0002-2739-7082")),
    person("Tomasz", "Kalinowski", role = "aut"),
    person("Will", "Landau", role = "aut"),
    person("Michael", "Lawrence", role = "aut"),
    person("Martin", "Maechler", role = "aut",
           comment = c(ORCID = "0000-0002-8685-9910")),
    person("Luke", "Tierney", role = "aut"),
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4757-117X"))
  )
Description: A new object oriented programming system designed to be a
    successor to S3 and S4. It includes formal class, generic, and method
    specification, and a limited form of multiple dispatch. It has been
    designed and implemented collaboratively by the R Consortium
    Object-Oriented Programming Working Group, which includes
    representatives from R-Core, 'Bioconductor', 'Posit'/'tidyverse', and
    the wider R community.
License: MIT + file LICENSE
URL: https://rconsortium.github.io/S7/,
        https://github.com/RConsortium/S7
BugReports: https://github.com/RConsortium/S7/issues
Depends: R (>= 3.5.0)
Imports: utils
Suggests: bench, callr, covr, knitr, methods, rmarkdown, testthat (>=
        3.2.0), tibble
VignetteBuilder: knitr
Config/build/compilation-database: true
Config/Needs/website: sloop
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Config/testthat/start-first: external-generic
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Author: Object-Oriented Programming Working Group [cph],
  Davis Vaughan [aut],
  Jim Hester [aut] (<https://orcid.org/0000-0002-2739-7082>),
  Tomasz Kalinowski [aut],
  Will Landau [aut],
  Michael Lawrence [aut],
  Martin Maechler [aut] (<https://orcid.org/0000-0002-8685-9910>),
  Luke Tierney [aut],
  Hadley Wickham [aut, cre] (<https://orcid.org/0000-0003-4757-117X>)
Maintainer: Hadley Wickham <hadley@posit.co>
Built: R 4.4.2; aarch64-apple-darwin24.3.0; 2025-03-03 05:39:50 UTC; unix
