# Generated by roxygen2: do not edit by hand

S3method("$",S7_object)
S3method("$<-",S7_object)
S3method("@<-",S7_object)
S3method("[",S7_object)
S3method("[<-",S7_object)
S3method("[[",S7_object)
S3method("[[<-",S7_object)
S3method("|",S7_class)
S3method(Ops,S7_object)
S3method(Ops,S7_super)
S3method(c,S7_class)
S3method(print,S7_S3_class)
S3method(print,S7_any)
S3method(print,S7_base_class)
S3method(print,S7_class)
S3method(print,S7_external_generic)
S3method(print,S7_generic)
S3method(print,S7_method)
S3method(print,S7_missing)
S3method(print,S7_object)
S3method(print,S7_property)
S3method(print,S7_super)
S3method(print,S7_union)
S3method(str,S7_S3_class)
S3method(str,S7_any)
S3method(str,S7_base_class)
S3method(str,S7_class)
S3method(str,S7_missing)
S3method(str,S7_object)
S3method(str,S7_property)
S3method(str,S7_super)
S3method(str,S7_union)
export("S7_data<-")
export("method<-")
export("prop<-")
export("props<-")
export(S4_register)
export(S7_class)
export(S7_data)
export(S7_dispatch)
export(S7_inherits)
export(S7_object)
export(as_class)
export(check_is_S7)
export(class_Date)
export(class_POSIXct)
export(class_POSIXlt)
export(class_POSIXt)
export(class_any)
export(class_atomic)
export(class_call)
export(class_character)
export(class_complex)
export(class_data.frame)
export(class_double)
export(class_environment)
export(class_expression)
export(class_factor)
export(class_formula)
export(class_function)
export(class_integer)
export(class_language)
export(class_list)
export(class_logical)
export(class_missing)
export(class_name)
export(class_numeric)
export(class_raw)
export(class_vector)
export(convert)
export(method)
export(method_explain)
export(methods_register)
export(new_S3_class)
export(new_class)
export(new_external_generic)
export(new_generic)
export(new_object)
export(new_property)
export(new_union)
export(prop)
export(prop_exists)
export(prop_names)
export(props)
export(set_props)
export(super)
export(valid_eventually)
export(valid_implicitly)
export(validate)
if (getRversion() < "4.3.0") export(`@`)
if (getRversion() >= "4.3.0" && !is.null(asNamespace("utils")$.AtNames)) S3method(utils::.AtNames,S7_object)
if (getRversion() >= "4.3.0") S3method(base::`@`, S7_object)
if (getRversion() >= "4.3.0") S3method(chooseOpsMethod, S7_object)
if (getRversion() >= "4.3.0") S3method(chooseOpsMethod, S7_super)
if (getRversion() >= "4.3.0") S3method(matrixOps, S7_object)
if (getRversion() >= "4.3.0") S3method(matrixOps, S7_super)
if (getRversion() >= "4.3.0") S3method(nameOfClass, S7_class, S7_class_name)
if (getRversion() >= "4.3.0") S3method(nameOfClass,S7_base_class)
importFrom(stats,setNames)
importFrom(utils,getFromNamespace)
importFrom(utils,globalVariables)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,packageName)
importFrom(utils,str)
useDynLib(S7, .registration = TRUE)
